@@ -167,7 +167,7 @@ oj_parse_mimic_dump_options(VALUE ropts, Options copts) {
     if (Qnil != (v = rb_hash_lookup(ropts, oj_ascii_only_sym))) {
 	// generate seems to assume anything except nil and false are true.
 	if (Qfalse == v) {
-	    copts->escape_mode = JXEsc; // JSONEsc;
+	    copts->escape_mode = JXEsc;
 	} else {
 	    copts->escape_mode = ASCIIEsc;
 	}
@@ -196,8 +196,8 @@ mimic_limit_arg(VALUE a) {
 static VALUE
 mimic_dump(int argc, VALUE *argv, VALUE self) {
     char		buf[4096];
-    struct _Out		out;
-    struct _Options	copts = oj_default_options;
+    struct _out		out;
+    struct _options	copts = oj_default_options;
     VALUE		rstr;
 
     copts.str_rx.head = NULL;
@@ -275,14 +275,10 @@ mimic_walk(VALUE key, VALUE obj, VALUE proc) {
 	    rb_yield(obj);
 	}
     } else {
-#if HAS_PROC_WITH_BLOCK
 	VALUE	args[1];
 
 	*args = obj;
 	rb_proc_call_with_block(proc, 1, args, Qnil);
-#else
-	rb_raise(rb_eNotImpError, "Calling a Proc with a block not supported in this version. Use func() {|x| } syntax instead.");
-#endif
     }
     return ST_CONTINUE;
 }
@@ -358,7 +354,7 @@ mimic_dump_load(int argc, VALUE *argv, VALUE self) {
 static VALUE
 mimic_generate_core(int argc, VALUE *argv, Options copts) {
     char	buf[4096];
-    struct _Out	out;
+    struct _out	out;
     VALUE	rstr;
 
     out.buf = buf;
@@ -418,7 +414,7 @@ mimic_generate_core(int argc, VALUE *argv, Options copts) {
  */
 VALUE
 oj_mimic_generate(int argc, VALUE *argv, VALUE self) {
-    struct _Options	copts = oj_default_options;
+    struct _options	copts = oj_default_options;
 
     copts.str_rx.head = NULL;
     copts.str_rx.tail = NULL;
@@ -436,7 +432,7 @@ oj_mimic_generate(int argc, VALUE *argv, VALUE self) {
  */
 VALUE
 oj_mimic_pretty_generate(int argc, VALUE *argv, VALUE self) {
-    struct _Options	copts = oj_default_options;
+    struct _options	copts = oj_default_options;
     VALUE		rargs[2];
     volatile VALUE	h;
 
@@ -485,7 +481,7 @@ oj_mimic_pretty_generate(int argc, VALUE *argv, VALUE self) {
 
 static VALUE
 mimic_parse_core(int argc, VALUE *argv, VALUE self, bool bang) {
-    struct _ParseInfo	pi;
+    struct _parseInfo	pi;
     VALUE		ropts;
     VALUE		args[1];
 
@@ -664,7 +660,7 @@ mimic_create_id(VALUE self) {
     return rb_str_new_cstr(oj_json_class);
 }
 
-static struct _Options	mimic_object_to_json_options = {
+static struct _options	mimic_object_to_json_options = {
     0,		// indent
     No,		// circular
     No,		// auto_define
@@ -686,6 +682,8 @@ static struct _Options	mimic_object_to_json_options = {
     No,		// create_ok
     No,		// allow_nan
     No,		// trace
+    0,		// integer_range_min
+    0,		// integer_range_max
     oj_json_class,// create_id
     10,		// create_id_len
     3,		// sec_prec
@@ -719,9 +717,9 @@ static struct _Options	mimic_object_to_json_options = {
 static VALUE
 mimic_object_to_json(int argc, VALUE *argv, VALUE self) {
     char		buf[4096];
-    struct _Out		out;
+    struct _out		out;
     VALUE		rstr;
-    struct _Options	copts = oj_default_options;
+    struct _options	copts = oj_default_options;
 
     copts.str_rx.head = NULL;
     copts.str_rx.tail = NULL;
