@@ -439,9 +439,9 @@ oj_set_obj_ivar(Val parent, Val kval, VALUE value) {
 	    rb_funcall(parent->val, rb_intern("set_backtrace"), 1, value);
 	}
     }
-#if USE_PTHREAD_MUTEX
+#if HAVE_LIBPTHREAD
     pthread_mutex_lock(&oj_cache_mutex);
-#elif USE_RB_MUTEX
+#else
     rb_mutex_lock(oj_cache_mutex);
 #endif
     if (0 == (var_id = oj_attr_hash_get(key, klen, &slot))) {
@@ -473,9 +473,9 @@ oj_set_obj_ivar(Val parent, Val kval, VALUE value) {
 	}
 	*slot = var_id;
     }
-#if USE_PTHREAD_MUTEX
+#if HAVE_LIBPTHREAD
     pthread_mutex_unlock(&oj_cache_mutex);
-#elif USE_RB_MUTEX
+#else
     rb_mutex_unlock(oj_cache_mutex);
 #endif
     rb_ivar_set(parent->val, var_id, value);
@@ -677,7 +677,7 @@ start_hash(ParseInfo pi) {
 }
 
 static void
-end_hash(struct _ParseInfo *pi) {
+end_hash(ParseInfo pi) {
     Val	parent = stack_peek(&pi->stack);
 
     if (Qnil == parent->val) {
@@ -765,7 +765,7 @@ oj_set_object_callbacks(ParseInfo pi) {
 
 VALUE
 oj_object_parse(int argc, VALUE *argv, VALUE self) {
-    struct _ParseInfo	pi;
+    struct _parseInfo	pi;
 
     parse_info_init(&pi);
     pi.options = oj_default_options;
@@ -782,7 +782,7 @@ oj_object_parse(int argc, VALUE *argv, VALUE self) {
 
 VALUE
 oj_object_parse_cstr(int argc, VALUE *argv, char *json, size_t len) {
-    struct _ParseInfo	pi;
+    struct _parseInfo	pi;
 
     parse_info_init(&pi);
     pi.options = oj_default_options;
