@@ -12,9 +12,9 @@
 #define OJ_INFINITY (1.0/0.0)
 
 // TBD keep static array of strings and functions to help with rails optimization
-typedef struct _Encoder {
-    struct _ROptTable	ropts;
-    struct _Options	opts;
+typedef struct _encoder {
+    struct _rOptTable	ropts;
+    struct _options	opts;
     VALUE		arg;
 } *Encoder;
 
@@ -28,7 +28,7 @@ static void	dump_rails_val(VALUE obj, int depth, Out out, bool as_ok);
 
 extern VALUE	Oj;
 
-static struct _ROptTable	ropts = { 0, 0, NULL };
+static struct _rOptTable	ropts = { 0, 0, NULL };
 
 static VALUE	encoder_class = Qnil;
 static bool	escape_html = true;
@@ -79,8 +79,8 @@ copy_opts(ROptTable src, ROptTable dest) {
     if (NULL == src->table) {
 	dest->table = NULL;
     } else {
-	dest->table = ALLOC_N(struct _ROpt, dest->alen);
-	memcpy(dest->table, src->table, sizeof(struct _ROpt) * dest->alen);
+	dest->table = ALLOC_N(struct _rOpt, dest->alen);
+	memcpy(dest->table, src->table, sizeof(struct _rOpt) * dest->alen);
     }
     return NULL;
 }
@@ -96,11 +96,9 @@ dump_attr_cb(ID key, VALUE value, Out out) {
     if (NULL == attr) {
 	attr = "";
     }
-#if HAS_EXCEPTION_MAGIC
     if (0 == strcmp("bt", attr) || 0 == strcmp("mesg", attr)) {
 	return ST_CONTINUE;
     }
-#endif
     assure_size(out, size);
     fill_indent(out, depth);
     if ('@' == *attr) {
@@ -164,8 +162,10 @@ dump_struct(VALUE obj, int depth, Out out, bool as_ok) {
     assure_size(out, 2);
     *out->cur++ = '{';
     for (i = 0; i < cnt; i++) {
-	name = rb_id2name(SYM2ID(rb_ary_entry(ma, i)));
-	len = (int)strlen(name);
+	volatile VALUE	s = rb_sym_to_s(rb_ary_entry(ma, i));
+
+	name = rb_string_value_ptr((VALUE*)&s);
+	len = (int)RSTRING_LEN(s);
 	assure_size(out, size + sep_len + 6);
 	if (0 < i) {
 	    *out->cur++ = ',';
@@ -214,9 +214,9 @@ dump_bigdecimal(VALUE obj, int depth, Out out, bool as_ok) {
     if ('I' == *str || 'N' == *str || ('-' == *str && 'I' == str[1])) {
 	oj_dump_nil(Qnil, depth, out, false);
     } else if (Yes == out->opts->bigdec_as_num) {
-	oj_dump_raw(str, RSTRING_LEN(rstr), out);
+	oj_dump_raw(str, (int)RSTRING_LEN(rstr), out);
     } else {
-	oj_dump_cstr(str, RSTRING_LEN(rstr), 0, 0, out);
+	oj_dump_cstr(str, (int)RSTRING_LEN(rstr), 0, 0, out);
     }
 }
 
@@ -301,22 +301,22 @@ dump_sec_nano(VALUE obj, time_t sec, long nsec, Out out) {
 
 static void
 dump_time(VALUE obj, int depth, Out out, bool as_ok) {
-    time_t	sec;
+    long long	sec;
     long long	nsec;
 
-#if HAS_RB_TIME_TIMESPEC
-    {
+#ifdef HAVE_RB_TIME_TIMESPEC
+    if (16 <= sizeof(struct timespec)) {
 	struct timespec	ts = rb_time_timespec(obj);
-	sec = ts.tv_sec;
+
+	sec = (long long)ts.tv_sec;
 	nsec = ts.tv_nsec;
+    } else {
+	sec = rb_num2ll(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
+	nsec = rb_num2ll(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
     }
 #else
     sec = rb_num2ll(rb_funcall2(obj, oj_tv_sec_id, 0, 0));
-#if HAS_NANO_TIME
     nsec = rb_num2ll(rb_funcall2(obj, oj_tv_nsec_id, 0, 0));
-#else
-    nsec = rb_num2ll(rb_funcall2(obj, oj_tv_usec_id, 0, 0)) * 1000;
-#endif
 #endif
     dump_sec_nano(obj, sec, nsec, out);
 }
@@ -338,12 +338,12 @@ static void
 dump_to_s(VALUE obj, int depth, Out out, bool as_ok) {
     volatile VALUE	rstr = rb_funcall(obj, oj_to_s_id, 0);
 
-    oj_dump_cstr(rb_string_value_ptr((VALUE*)&rstr), RSTRING_LEN(rstr), 0, 0, out);
+    oj_dump_cstr(rb_string_value_ptr((VALUE*)&rstr), (int)RSTRING_LEN(rstr), 0, 0, out);
 }
 
 static ID	parameters_id = 0;
 
-typedef struct _StrLen {
+typedef struct _strLen {
     const char	*str;
     int		len;
 } *StrLen;
@@ -366,7 +366,7 @@ columns_array(VALUE rcols, int *ccnt) {
     int			cnt = (int)RARRAY_LEN(rcols);
     
     *ccnt = cnt;
-    cols = ALLOC_N(struct _StrLen, cnt);
+    cols = ALLOC_N(struct _strLen, cnt);
     for (i = 0, cp = cols; i < cnt; i++, cp++) {
 	v = rb_ary_entry(rcols, i);
 	if (T_STRING != rb_type(v)) {
@@ -451,7 +451,7 @@ dump_activerecord_result(VALUE obj, int depth, Out out, bool as_ok) {
     out->argc = 0;
     cols = columns_array(rb_ivar_get(obj, columns_id), &ccnt);
     rows = rb_ivar_get(obj, rows_id);
-    rcnt = RARRAY_LEN(rows);
+    rcnt = (int)RARRAY_LEN(rows);
     assure_size(out, 2);
     *out->cur++ = '[';
     if (out->opts->dump_opts.use) {
@@ -504,7 +504,7 @@ dump_activerecord_result(VALUE obj, int depth, Out out, bool as_ok) {
     *out->cur++ = ']';
 }
 
-typedef struct _NamedFunc {
+typedef struct _namedFunc {
     const char	*name;
     DumpFunc	func;
 } *NamedFunc;
@@ -527,15 +527,11 @@ dump_as_json(VALUE obj, int depth, Out out, bool as_ok) {
     }
     // Some classes elect to not take an options argument so check the arity
     // of as_json.
-#if HAS_METHOD_ARITY
     if (0 == rb_obj_method_arity(obj, oj_as_json_id)) {
 	ja = rb_funcall(obj, oj_as_json_id, 0);
     } else {
 	ja = rb_funcall2(obj, oj_as_json_id, out->argc, out->argv);
     }
-#else
-    ja = rb_funcall2(obj, oj_as_json_id, out->argc, out->argv);
-#endif
     if (Yes == out->opts->trace) {
 	oj_trace("as_json", obj, __FILE__, __LINE__, depth + 1, TraceRubyOut);
     }
@@ -565,7 +561,7 @@ dump_regexp(VALUE obj, int depth, Out out, bool as_ok) {
     dump_as_string(obj, depth, out, as_ok);
 }
 
-static struct _NamedFunc	dump_map[] = {
+static struct _namedFunc	dump_map[] = {
     { "ActionController::Parameters", dump_actioncontroller_parameters },
     { "ActiveRecord::Result", dump_activerecord_result },
     { "ActiveSupport::TimeWithZone", dump_timewithzone },
@@ -599,12 +595,12 @@ create_opt(ROptTable rot, VALUE clas) {
     rot->len++;
     if (NULL == rot->table) {
 	rot->alen = 256;
-	rot->table = ALLOC_N(struct _ROpt, rot->alen);
-	memset(rot->table, 0, sizeof(struct _ROpt) * rot->alen);
+	rot->table = ALLOC_N(struct _rOpt, rot->alen);
+	memset(rot->table, 0, sizeof(struct _rOpt) * rot->alen);
     } else if (rot->alen <= rot->len) {
 	rot->alen *= 2;
-	REALLOC_N(rot->table, struct _ROpt, rot->alen);
-	memset(rot->table + olen, 0, sizeof(struct _ROpt) * olen);
+	REALLOC_N(rot->table, struct _rOpt, rot->alen);
+	memset(rot->table + olen, 0, sizeof(struct _rOpt) * olen);
     }
     if (0 == olen) {
 	ro = rot->table;
@@ -615,7 +611,7 @@ create_opt(ROptTable rot, VALUE clas) {
 	
 	for (i = 0, ro = rot->table; i < olen; i++, ro++) {
 	    if (clas < ro->clas) {
-		memmove(ro + 1, ro, sizeof(struct _ROpt) * (olen - i));
+		memmove(ro + 1, ro, sizeof(struct _rOpt) * (olen - i));
 		break;
 	    }
 	}
@@ -682,7 +678,7 @@ encoder_mark(void *ptr) {
  */
 static VALUE
 encoder_new(int argc, VALUE *argv, VALUE self) {
-    Encoder	e = ALLOC(struct _Encoder);
+    Encoder	e = ALLOC(struct _encoder);
 
     e->opts = oj_default_options;
     e->arg = Qnil;
@@ -828,7 +824,7 @@ rails_mimic_json(VALUE self) {
 	json = rb_define_module("JSON");
     }
     oj_mimic_json_methods(json);
-    oj_default_options.mode = RailsMode;
+    //oj_default_options.mode = RailsMode;
 
     return Qnil;
 }
@@ -896,7 +892,7 @@ rails_optimized(VALUE self, VALUE clas) {
     return (ro->on) ? Qtrue : Qfalse;
 }
 
-typedef struct _OO {
+typedef struct _oo {
     Out		out;
     VALUE	obj;
 } *OO;
@@ -913,10 +909,10 @@ protect_dump(VALUE ov) {
 static VALUE
 encode(VALUE obj, ROptTable ropts, Options opts, int argc, VALUE *argv) {
     char		buf[4096];
-    struct _Out		out;
-    struct _Options	copts = *opts;
+    struct _out		out;
+    struct _options	copts = *opts;
     volatile VALUE	rstr = Qnil;
-    struct _OO		oo;
+    struct _oo		oo;
     int			line = 0;
 
     oo.out = &out;
@@ -925,7 +921,7 @@ encode(VALUE obj, ROptTable ropts, Options opts, int argc, VALUE *argv) {
     copts.str_rx.tail = NULL;
     copts.mode = RailsMode;
     if (escape_html) {
-	copts.escape_mode = JXEsc;
+	copts.escape_mode = RailsXEsc;
     } else {
 	copts.escape_mode = RailsEsc;
     }
@@ -1022,16 +1018,11 @@ rails_encode(int argc, VALUE *argv, VALUE self) {
 
 static VALUE
 rails_use_standard_json_time_format(VALUE self, VALUE state) {
-    switch (state) {
-    case Qtrue:
-    case Qfalse:
-	break;
-    case Qnil:
+    if (Qtrue == state || Qfalse == state) {
+    } else if (Qnil == state) {
 	state = Qfalse;
-	break;
-    default:
+    } else {
 	state = Qtrue;
-	break;
     }
     rb_iv_set(self, "@use_standard_json_time_format", state);
     xml_time = Qtrue == state;
@@ -1494,7 +1485,7 @@ oj_dump_rails_val(VALUE obj, int depth, Out out) {
     out->opts->str_rx.head = NULL;
     out->opts->str_rx.tail = NULL;
     if (escape_html) {
-	out->opts->escape_mode = JXEsc;
+	out->opts->escape_mode = RailsXEsc;
     } else {
 	out->opts->escape_mode = RailsEsc;
     }
