@@ -94,7 +94,7 @@ stream_writer_new(int argc, VALUE *argv, VALUE self) {
     } else {
 	rb_raise(rb_eArgError, "expected an IO Object.");
     }
-    sw = ALLOC(struct _StreamWriter);
+    sw = ALLOC(struct _streamWriter);
     if (2 == argc && T_HASH == rb_type(argv[1])) {
 	volatile VALUE	v;
 	int		buf_size = 0;
@@ -271,7 +271,7 @@ stream_writer_push_json(int argc, VALUE *argv, VALUE self) {
 	oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), 0);
 	break;
     case 2:
-	if (Qnil == argv[0]) {
+	if (Qnil == argv[1]) {
 	    oj_str_writer_push_json((StrWriter)DATA_PTR(self), StringValuePtr(*argv), 0);
 	} else {
 	    rb_check_type(argv[1], T_STRING);
