@@ -28,6 +28,8 @@
  * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
+#include <string.h>
+
 #include "oj.h"
 #include "val_stack.h"
 
@@ -39,9 +41,9 @@ mark(void *ptr) {
     if (0 == ptr) {
 	return;
     }
-#if USE_PTHREAD_MUTEX
+#if HAVE_LIBPTHREAD
     pthread_mutex_lock(&stack->mutex);
-#elif USE_RB_MUTEX
+#else
     rb_mutex_lock(stack->mutex);
     rb_gc_mark(stack->mutex);
 #endif
@@ -53,22 +55,26 @@ mark(void *ptr) {
 	    rb_gc_mark(v->key_val);
 	}
     }
-#if USE_PTHREAD_MUTEX
+#if HAVE_LIBPTHREAD
     pthread_mutex_unlock(&stack->mutex);
-#elif USE_RB_MUTEX
+#else
     rb_mutex_unlock(stack->mutex);
 #endif
 }
 
 VALUE
 oj_stack_init(ValStack stack) {
-#if USE_PTHREAD_MUTEX
-    pthread_mutex_init(&stack->mutex, 0);
-#elif USE_RB_MUTEX
+#if HAVE_LIBPTHREAD
+    int	err;
+    
+    if (0 != (err = pthread_mutex_init(&stack->mutex, 0))) {
+	rb_raise(rb_eException, "failed to initialize a mutex. %s", strerror(err));
+    }
+#else
     stack->mutex = rb_mutex_new();
 #endif
     stack->head = stack->base;
-    stack->end = stack->base + sizeof(stack->base) / sizeof(struct _Val);
+    stack->end = stack->base + sizeof(stack->base) / sizeof(struct _val);
     stack->tail = stack->head;
     stack->head->val = Qundef;
     stack->head->key = 0;
